Module: Chewy::Stash

Defined in:
lib/chewy/stash.rb

Overview

This class is the main storage for Chewy service data, Now index raw specifications are stored in the ‘chewy_specifications` index. Journal entries are stored in `chewy_journal`

Defined Under Namespace

Classes: Journal, Specification